@@ -18,7 +18,7 @@ class IsInf(Base):
             outputs=["y"],
         )
 
-        x = np.array([-1.2, np.nan, np.inf, 2.8, np.NINF, np.inf], dtype=np.float32)
+        x = np.array([-1.2, np.nan, np.inf, 2.8, -np.inf, np.inf], dtype=np.float32)
         y = np.isinf(x)
         expect(node, inputs=[x], outputs=[y], name="test_isinf")
 
@@ -28,7 +28,7 @@ class IsInf(Base):
             "IsInf", inputs=["x"], outputs=["y"], detect_negative=0
         )
 
-        x = np.array([-1.7, np.nan, np.inf, 3.6, np.NINF, np.inf], dtype=np.float32)
+        x = np.array([-1.7, np.nan, np.inf, 3.6, -np.inf, np.inf], dtype=np.float32)
         y = np.isposinf(x)
         expect(node, inputs=[x], outputs=[y], name="test_isinf_positive")
 
@@ -38,6 +38,18 @@ class IsInf(Base):
             "IsInf", inputs=["x"], outputs=["y"], detect_positive=0
         )
 
-        x = np.array([-1.7, np.nan, np.inf, -3.6, np.NINF, np.inf], dtype=np.float32)
+        x = np.array([-1.7, np.nan, np.inf, -3.6, -np.inf, np.inf], dtype=np.float32)
         y = np.isneginf(x)
         expect(node, inputs=[x], outputs=[y], name="test_isinf_negative")
+
+    @staticmethod
+    def export_infinity_float16() -> None:
+        node = onnx.helper.make_node(
+            "IsInf",
+            inputs=["x"],
+            outputs=["y"],
+        )
+
+        x = np.array([-1.2, np.nan, np.inf, 2.8, -np.inf, np.inf], dtype=np.float16)
+        y = np.isinf(x)
+        expect(node, inputs=[x], outputs=[y], name="test_isinf_float16")
