@@ -10,24 +10,11 @@ import os
 import re
 import shutil
 import sys
-import tarfile
 import tempfile
 import time
 import unittest
 from collections import defaultdict
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Iterable,
-    List,
-    Optional,
-    Pattern,
-    Sequence,
-    Set,
-    Type,
-    Union,
-)
+from typing import Any, Callable, Iterable, Pattern, Sequence
 from urllib.request import urlretrieve
 
 import numpy as np
@@ -67,19 +54,23 @@ def retry_execute(times: int) -> Callable[[Callable[..., Any]], Callable[..., An
 
 class Runner:
     def __init__(
-        self, backend: Type[Backend], parent_module: Optional[str] = None
+        self,
+        backend: type[Backend],
+        parent_module: str | None = None,
+        test_kwargs: dict | None = None,
     ) -> None:
         self.backend = backend
         self._parent_module = parent_module
-        self._include_patterns: Set[Pattern[str]] = set()
-        self._exclude_patterns: Set[Pattern[str]] = set()
-        self._xfail_patterns: Set[Pattern[str]] = set()
+        self._include_patterns: set[Pattern[str]] = set()
+        self._exclude_patterns: set[Pattern[str]] = set()
+        self._xfail_patterns: set[Pattern[str]] = set()
+        self._test_kwargs: dict = test_kwargs or {}
 
         # This is the source of the truth of all test functions.
         # Properties `test_cases`, `test_suite` and `tests` will be
         # derived from it.
         # {category: {name: func}}
-        self._test_items: Dict[str, Dict[str, TestItem]] = defaultdict(dict)
+        self._test_items: dict[str, dict[str, TestItem]] = defaultdict(dict)
 
         for rt in load_model_tests(kind="node"):
             self._add_model_test(rt, "Node")
@@ -96,7 +87,7 @@ class Runner:
         for ot in load_model_tests(kind="pytorch-operator"):
             self._add_model_test(ot, "PyTorchOperator")
 
-    def _get_test_case(self, name: str) -> Type[unittest.TestCase]:
+    def _get_test_case(self, name: str) -> type[unittest.TestCase]:
         test_case = type(str(name), (unittest.TestCase,), {})
         if self._parent_module:
             test_case.__module__ = self._parent_module
@@ -123,8 +114,8 @@ class Runner:
         return self
 
     @property
-    def _filtered_test_items(self) -> Dict[str, Dict[str, TestItem]]:
-        filtered: Dict[str, Dict[str, TestItem]] = {}
+    def _filtered_test_items(self) -> dict[str, dict[str, TestItem]]:
+        filtered: dict[str, dict[str, TestItem]] = {}
         for category, items_map in self._test_items.items():
             filtered[category] = {}
             for name, item in items_map.items():
@@ -144,9 +135,8 @@ class Runner:
         return filtered
 
     @property
-    def test_cases(self) -> Dict[str, Type[unittest.TestCase]]:
-        """
-        List of test cases to be applied on the parent scope
+    def test_cases(self) -> dict[str, type[unittest.TestCase]]:
+        """List of test cases to be applied on the parent scope
         Example usage:
             globals().update(BackendTest(backend).test_cases)
         """
@@ -161,8 +151,7 @@ class Runner:
 
     @property
     def test_suite(self) -> unittest.TestSuite:
-        """
-        TestSuite that can be run by TestRunner
+        """TestSuite that can be run by TestRunner
         Example usage:
             unittest.TextTestRunner().run(BackendTest(backend).test_suite)
         """
@@ -175,9 +164,8 @@ class Runner:
 
     # For backward compatibility (we used to expose `.tests`)
     @property
-    def tests(self) -> Type[unittest.TestCase]:
-        """
-        One single unittest.TestCase that hosts all the test functions
+    def tests(self) -> type[unittest.TestCase]:
+        """One single unittest.TestCase that hosts all the test functions
         Example usage:
             onnx_backend_tests = BackendTest(backend).tests
         """
@@ -196,15 +184,35 @@ class Runner:
         outputs: Sequence[Any],
         rtol: float,
         atol: float,
+        model_dir: str | None = None,
     ) -> None:
-        np.testing.assert_equal(len(outputs), len(ref_outputs))
+        try:
+            np.testing.assert_equal(len(outputs), len(ref_outputs))
+        except TypeError as e:
+            raise TypeError(
+                f"Unable to compare expected type {type(ref_outputs)} "
+                f"and runtime type {type(outputs)} (known test={model_dir or '?'!r})"
+            ) from e
         for i in range(len(outputs)):
             if isinstance(outputs[i], (list, tuple)):
+                if not isinstance(ref_outputs[i], (list, tuple)):
+                    raise AssertionError(
+                        f"Unexpected type {type(outputs[i])} for outputs[{i}]. Expected "
+                        f"type is {type(ref_outputs[i])} (known test={model_dir or '?'!r})."
+                    )
                 for j in range(len(outputs[i])):
                     cls.assert_similar_outputs(
-                        ref_outputs[i][j], outputs[i][j], rtol, atol
+                        ref_outputs[i][j],
+                        outputs[i][j],
+                        rtol,
+                        atol,
+                        model_dir=model_dir,
                     )
             else:
+                if not np.issubdtype(ref_outputs[i].dtype, np.number):
+                    if ref_outputs[i].tolist() != outputs[i].tolist():
+                        raise AssertionError(f"{ref_outputs[i]} != {outputs[i]}")
+                    continue
                 np.testing.assert_equal(outputs[i].dtype, ref_outputs[i].dtype)
                 if ref_outputs[i].dtype == object:  # type: ignore[attr-defined]
                     np.testing.assert_array_equal(outputs[i], ref_outputs[i])
@@ -229,8 +237,7 @@ class Runner:
             )
             urlretrieve(model_test.url, download_file.name)
             print("Done")
-            with tarfile.open(download_file.name) as t:
-                t.extractall(models_dir)
+            onnx.utils._extract_model_safe(download_file.name, models_dir)
         except Exception as e:
             print(f"Failed to prepare data for model {model_test.model_name}: {e}")
             raise
@@ -266,8 +273,9 @@ class Runner:
         category: str,
         test_name: str,
         test_func: Callable[..., Any],
-        report_item: List[Optional[Union[ModelProto, NodeProto]]],
+        report_item: list[ModelProto | NodeProto | None],
         devices: Iterable[str] = ("CPU", "CUDA"),
+        **kwargs: Any,
     ) -> None:
         # We don't prepend the 'test_' prefix to improve greppability
         if not test_name.startswith("test_"):
@@ -285,9 +293,10 @@ class Runner:
                 f"Backend doesn't support device {device}",
             )
             @functools.wraps(test_func)
-            def device_test_func(*args: Any, **kwargs: Any) -> Any:
+            def device_test_func(*args: Any, **device_test_kwarg: Any) -> Any:
                 try:
-                    return test_func(*args, device=device, **kwargs)
+                    merged_kwargs = {**kwargs, **device_test_kwarg}
+                    return test_func(*args, device, **merged_kwargs)
                 except BackendIsNotSupposedToImplementIt as e:
                     # hacky verbose reporting
                     if "-v" in sys.argv or "--verbose" in sys.argv:
@@ -304,9 +313,7 @@ class Runner:
     def generate_dummy_data(
         x: ValueInfoProto, seed: int = 0, name: str = "", random: bool = False
     ) -> np.ndarray:
-        """
-        Generates a random tensor based on the input definition.
-        """
+        """Generates a random tensor based on the input definition."""
         if not x.type.tensor_type:
             raise NotImplementedError(
                 f"Input expected to have tensor type. "
@@ -330,9 +337,9 @@ class Runner:
     def _add_model_test(self, model_test: TestCase, kind: str) -> None:
         # model is loaded at runtime, note sometimes it could even
         # never loaded if the test skipped
-        model_marker: List[Optional[Union[ModelProto, NodeProto]]] = [None]
+        model_marker: list[ModelProto | NodeProto | None] = [None]
 
-        def run(test_self: Any, device: str) -> None:
+        def run(test_self: Any, device: str, **kwargs) -> None:
             if model_test.url is not None and model_test.url.startswith(
                 "onnx/backend/test/data/light/"
             ):
@@ -379,7 +386,7 @@ class Runner:
             ):
                 raise unittest.SkipTest("Not compatible with backend")
 
-            prepared_model = self.backend.prepare(model, device)
+            prepared_model = self.backend.prepare(model, device, **kwargs)
             assert prepared_model is not None
 
             if use_dummy:
@@ -396,7 +403,7 @@ class Runner:
                 if not os.path.exists(test_data_set):
                     os.mkdir(test_data_set)
                 feeds = {}
-                inits = set(i.name for i in onx.graph.initializer)
+                inits = {i.name for i in onx.graph.initializer}
                 n_input = 0
                 inputs = []
                 for i in range(len(onx.graph.input)):
@@ -443,9 +450,16 @@ class Runner:
                     test_data = np.load(test_data_npz, encoding="bytes")
                     inputs = list(test_data["inputs"])
                     outputs = list(prepared_model.run(inputs))
-                    ref_outputs = test_data["outputs"]
+                    ref_outputs = tuple(
+                        np.array(x) if not isinstance(x, (list, dict)) else x
+                        for f in test_data["outputs"]
+                    )
                     self.assert_similar_outputs(
-                        ref_outputs, outputs, rtol=model_test.rtol, atol=model_test.atol
+                        ref_outputs,
+                        outputs,
+                        rtol=model_test.rtol,
+                        atol=model_test.atol,
+                        model_dir=model_dir,
                     )
 
             for test_data_dir in glob.glob(os.path.join(model_dir, "test_data_set*")):
@@ -465,15 +479,28 @@ class Runner:
                     )
                 outputs = list(prepared_model.run(inputs))
                 self.assert_similar_outputs(
-                    ref_outputs, outputs, rtol=model_test.rtol, atol=model_test.atol
+                    ref_outputs,
+                    outputs,
+                    rtol=model_test.rtol,
+                    atol=model_test.atol,
+                    model_dir=model_dir,
                 )
 
-        self._add_test(kind + "Model", model_test.name, run, model_marker)
+        if model_test.name in self._test_kwargs:
+            self._add_test(
+                kind + "Model",
+                model_test.name,
+                run,
+                model_marker,
+                **self._test_kwargs[model_test.name],
+            )
+        else:
+            self._add_test(kind + "Model", model_test.name, run, model_marker)
 
     def _load_proto(
         self,
         proto_filename: str,
-        target_list: List[Union[np.ndarray, List[Any]]],
+        target_list: list[np.ndarray | list[Any]],
         model_type_proto: TypeProto,
     ) -> None:
         with open(proto_filename, "rb") as f:
@@ -485,7 +512,9 @@ class Runner:
             elif model_type_proto.HasField("tensor_type"):
                 tensor = onnx.TensorProto()
                 tensor.ParseFromString(protobuf_content)
-                target_list.append(numpy_helper.to_array(tensor))
+                t = numpy_helper.to_array(tensor)
+                assert isinstance(t, np.ndarray)
+                target_list.append(t)
             elif model_type_proto.HasField("optional_type"):
                 optional = onnx.OptionalProto()
                 optional.ParseFromString(protobuf_content)
