@@ -9,7 +9,6 @@ import hashlib
 import json
 import os
 import sys
-import tarfile
 from io import BytesIO
 from os.path import join
 from typing import IO, Any, Dict, List, Optional, Set, Tuple, cast
@@ -27,8 +26,7 @@ else:
 
 
 class ModelInfo:
-    """
-    A class to represent a model's property and metadata in the ONNX Hub.
+    """A class to represent a model's property and metadata in the ONNX Hub.
     It extracts model name, path, sha, tags, etc. from the passed in raw_model_info dict.
 
     Attributes:
@@ -41,8 +39,9 @@ class ModelInfo:
     """
 
     def __init__(self, raw_model_info: Dict[str, Any]) -> None:
-        """
-        Parameters:
+        """Initializer.
+
+        Args:
             raw_model_info: A JSON dict containing the model info.
         """
         self.model = cast(str, raw_model_info["model"])
@@ -68,28 +67,26 @@ class ModelInfo:
 
 
 def set_dir(new_dir: str) -> None:
-    """
-    Sets the current ONNX hub cache location
+    """Sets the current ONNX hub cache location.
 
-    :param new_dir: location of new model hub cache
+    Args:
+        new_dir: Location of new model hub cache.
     """
-    global _ONNX_HUB_DIR  # pylint: disable=global-statement
+    global _ONNX_HUB_DIR  # noqa: PLW0603
     _ONNX_HUB_DIR = new_dir
 
 
 def get_dir() -> str:
-    """
-    Gets the current ONNX hub cache location
+    """Gets the current ONNX hub cache location.
 
-    :return: The location of the ONNX hub model cache
+    Returns:
+        The location of the ONNX hub model cache.
     """
     return _ONNX_HUB_DIR
 
 
 def _parse_repo_info(repo: str) -> Tuple[str, str, str]:
-    """
-    Gets the repo owner, name and ref from a repo specification string.
-    """
+    """Gets the repo owner, name and ref from a repo specification string."""
     repo_owner = repo.split(":")[0].split("/")[0]
     repo_name = repo.split(":")[0].split("/")[1]
     if ":" in repo:
@@ -100,8 +97,7 @@ def _parse_repo_info(repo: str) -> Tuple[str, str, str]:
 
 
 def _verify_repo_ref(repo: str) -> bool:
-    """
-    Verifies whether the given model repo can be trusted.
+    """Verifies whether the given model repo can be trusted.
     A model repo can be trusted if it matches onnx/models:main.
     """
     repo_owner, repo_name, repo_ref = _parse_repo_info(repo)
@@ -109,13 +105,16 @@ def _verify_repo_ref(repo: str) -> bool:
 
 
 def _get_base_url(repo: str, lfs: bool = False) -> str:
-    """
-    Gets the base github url from a repo specification string
+    """Gets the base github url from a repo specification string.
 
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param lfs: whether the url is for downloading lfs models
-    :return: the base github url for downloading
+    Args:
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main".
+        lfs: Whether the url is for downloading lfs models.
+
+    Returns:
+        The base github url for downloading.
     """
     repo_owner, repo_name, repo_ref = _parse_repo_info(repo)
 
@@ -125,11 +124,11 @@ def _get_base_url(repo: str, lfs: bool = False) -> str:
 
 
 def _download_file(url: str, file_name: str) -> None:
-    """
-    Downloads the file with specifed file_name from the url
+    """Downloads the file with specified file_name from the url.
 
-    :param url: a url of download link
-    :param file_name: a specified file name for the downloaded file
+    Args:
+        url: A url of download link.
+        file_name: A specified file name for the downloaded file.
     """
     chunk_size = 16384  # 1024 * 16
     with urlopen(url) as response, open(file_name, "wb") as f:
@@ -146,14 +145,19 @@ def list_models(
     model: Optional[str] = None,
     tags: Optional[List[str]] = None,
 ) -> List[ModelInfo]:
-    """
-    Gets the list of model info consistent with a given name and tags
+    """Gets the list of model info consistent with a given name and tags
 
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param model: The name of the model to search for. If `None`, will return all models with matching tags.
-    :param tags: A list of tags to filter models by. If `None`, will return all models with matching name.
-    :return: list of ModelInfo
+    Args:
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        model: The name of the model to search for. If `None`, will
+            return all models with matching tags.
+        tags: A list of tags to filter models by. If `None`, will return
+            all models with matching name.
+
+    Returns:
+        ``ModelInfo``s.
     """
     base_url = _get_base_url(repo)
     manifest_url = base_url + "ONNX_HUB_MANIFEST.json"
@@ -188,14 +192,19 @@ def list_models(
 def get_model_info(
     model: str, repo: str = "onnx/models:main", opset: Optional[int] = None
 ) -> ModelInfo:
-    """
-    Gets the model info matching the given name and opset.
+    """Gets the model info matching the given name and opset.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to get. The default of `None` will return the model with largest opset.
-    :return: ModelInfo
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to get. The default of `None` will
+            return the model with largest opset.
+
+    Returns:
+        ``ModelInfo``.
     """
     matching_models = list_models(repo, model)
     if not matching_models:
@@ -205,7 +214,7 @@ def get_model_info(
         selected_models = sorted(matching_models, key=lambda m: -m.opset)
     else:
         selected_models = [m for m in matching_models if m.opset == opset]
-        if len(selected_models) == 0:
+        if not selected_models:
             valid_opsets = [m.opset for m in matching_models]
             raise AssertionError(
                 f"{model} has no version with opset {opset}. Valid opsets: {valid_opsets}"
@@ -220,16 +229,23 @@ def load(
     force_reload: bool = False,
     silent: bool = False,
 ) -> Optional[onnx.ModelProto]:
-    """
-    Downloads a model by name from the onnx model hub
+    """Downloads a model by name from the onnx model hub.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: ModelProto or None
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        ModelProto or None
     """
     selected_model = get_model_info(model, repo, opset)
     local_model_path_arr = selected_model.model_path.split("/")
@@ -262,10 +278,10 @@ def load(
         downloaded_sha = hashlib.sha256(model_bytes).hexdigest()
         if not downloaded_sha == selected_model.model_sha:
             raise AssertionError(
-                (
-                    f"The cached model {selected_model.model} has SHA256 {downloaded_sha} while checksum should be {selected_model.model_sha}."
-                    + "The model in the hub may have been updated. Use force_reload to download the model from the model hub."
-                )
+                f"The cached model {selected_model.model} has SHA256 {downloaded_sha} "
+                f"while checksum should be {selected_model.model_sha}. "
+                "The model in the hub may have been updated. Use force_reload to "
+                "download the model from the model hub."
             )
 
     return onnx.load(cast(IO[bytes], BytesIO(model_bytes)))
@@ -278,16 +294,24 @@ def download_model_with_test_data(
     force_reload: bool = False,
     silent: bool = False,
 ) -> Optional[str]:
-    """
-    Downloads a model along with test data by name from the onnx model hub and returns the directory to which the files have been extracted.
+    """Downloads a model along with test data by name from the onnx model hub and returns the directory to which the files have been extracted.
+    Users are responsible for making sure the model comes from a trusted source, and the data is safe to be extracted.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: str or None
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        str or None
     """
     selected_model = get_model_info(model, repo, opset)
 
@@ -331,18 +355,20 @@ def download_model_with_test_data(
         downloaded_sha = hashlib.sha256(model_with_data_bytes).hexdigest()
         if not downloaded_sha == model_with_data_sha:
             raise AssertionError(
-                (
-                    f"The cached model {selected_model.model} has SHA256 {downloaded_sha} while checksum should be {model_with_data_sha}."
-                    + "The model in the hub may have been updated. Use force_reload to download the model from the model hub."
-                )
+                f"The cached model {selected_model.model} has SHA256 {downloaded_sha} "
+                f"while checksum should be {model_with_data_sha}. "
+                "The model in the hub may have been updated. Use force_reload to "
+                "download the model from the model hub."
             )
 
-    with tarfile.open(local_model_with_data_path) as model_with_data_zipped:
-        # FIXME: Avoid index manipulation with magic numbers
-        local_model_with_data_dir_path = local_model_with_data_path[
-            0 : len(local_model_with_data_path) - 7
-        ]
-        model_with_data_zipped.extractall(local_model_with_data_dir_path)
+    # FIXME: Avoid index manipulation with magic numbers,
+    # remove ".tar.gz"
+    local_model_with_data_dir_path = local_model_with_data_path[
+        0 : len(local_model_with_data_path) - 7
+    ]
+    onnx.utils._extract_model_safe(
+        local_model_with_data_path, local_model_with_data_dir_path
+    )
     model_with_data_path = (
         local_model_with_data_dir_path
         + "/"
@@ -361,17 +387,27 @@ def load_composite_model(
     force_reload: bool = False,
     silent: bool = False,
 ) -> Optional[onnx.ModelProto]:
-    """
-    Builds a composite model including data preprocessing by downloading a network and a preprocessing model
+    """Builds a composite model including data preprocessing by downloading a network and a preprocessing model
     and combine it into a single model
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: ModelProto or None
+    Args:
+        network_model: The name of the onnx model in the manifest.
+        preprocessing_model: The name of the preprocessing model.
+        network_repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        preprocessing_repo: The location of the proprocessing model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        ModelProto or None
     """
     preprocessing = load(
         preprocessing_model, preprocessing_repo, opset, force_reload, silent
@@ -433,9 +469,10 @@ def load_composite_model(
         preprocessing.ir_version = network.ir_version
         onnx.checker.check_model(preprocessing)
 
-    io_map = []
-    for out_entry, in_entry in zip(preprocessing.graph.output, network.graph.input):
-        io_map.append((out_entry.name, in_entry.name))
+    io_map = [
+        (out_entry.name, in_entry.name)
+        for out_entry, in_entry in zip(preprocessing.graph.output, network.graph.input)
+    ]
 
     model_with_preprocessing = onnx.compose.merge_models(
         preprocessing, network, io_map=io_map
